@propertyWrapper
public class State<Value> {
    private var _originalValue: Value
    private var _wrappedValue: Value
    public var wrappedValue: Value {
        get { _wrappedValue }
        set {
            let oldValue = _wrappedValue
            _wrappedValue = newValue
            listeners.forEach { $0(oldValue, newValue) }
        }
    }
    
    public var projectedValue: State<Value> { self }

    public init(initialValue value: Value) {
        _originalValue = value
        _wrappedValue = value
    }
    
    public func reset() {
        let oldValue = _wrappedValue
        _wrappedValue = _originalValue
        listeners.forEach { $0(oldValue, _wrappedValue) }
    }
    
    public typealias Listener = (_ old: Value, _ new: Value) -> Void
    public typealias SimpleListener = (_ value: Value) -> Void
    
    private var listeners: [Listener] = []
    
    public func listen(_ listener: @escaping Listener) {
        listeners.append(listener)
    }
    
    public func listen(_ listener: @escaping SimpleListener) {
        listeners.append({ _, new in listener(new) })
    }
}
